@@ -14,13 +14,13 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import javax.swing.Icon;
+import uk.ac.starlink.table.ValueInfo;
 import uk.ac.starlink.ttools.plot.PdfGraphicExporter;
 import uk.ac.starlink.ttools.plot.Picture;
 import uk.ac.starlink.ttools.plot.Range;
@@ -28,7 +28,6 @@ import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
 import uk.ac.starlink.ttools.plot2.data.DataStore;
 import uk.ac.starlink.ttools.plot2.data.TupleSequence;
-import uk.ac.starlink.ttools.plot2.layer.BinList;
 import uk.ac.starlink.ttools.plot2.paper.PaperType;
 
 /**
@@ -63,6 +62,9 @@ public class PlotUtil {
         public int getIntValue( int icol ) {
             throw new IllegalStateException();
         }
+        public long getLongValue( int icol ) {
+            throw new IllegalStateException();
+        }
         public boolean getBooleanValue( int icol ) {
             throw new IllegalStateException();
         }
@@ -532,16 +534,49 @@ public class PlotUtil {
     }
 
     /**
-     * Modifies a supplied range object by submitting the values in the
-     * bins of a given BinList.Result.
-     * 
-     * @param  range  range to extend
-     * @param  binResult  bin data
+     * Attempts to return input-level metadata about an aux value that may
+     * be referenced in a given list of plot layers.
+     *
+     * @param   layers   list of layers
+     * @param   scale    aux item of interest
+     * @return   input metadata for scale, or null if nothing suitable can
+     *           be found
      */
-    public static void extendRange( Range range, BinList.Result binResult ) {
-        for ( Iterator<Long> it = binResult.indexIterator(); it.hasNext(); ) {
-            range.submit( binResult.getBinValue( it.next().longValue() ) );
+    public static ValueInfo getScaleInfo( PlotLayer[] layers, AuxScale scale ) {
+        for ( PlotLayer layer : layers ) {
+            AuxReader rdr = layer.getAuxRangers().get( scale );
+            if ( rdr != null ) {
+                ValueInfo info = rdr.getAxisInfo( layer.getDataSpec() );
+                if ( info != null ) {
+                    return info;
+                }
+            }
         }
+        return null;
+    }
+
+    /**
+     * Attempts to return a suitable label for an aux axis that may
+     * be referenced in a given list of plot layers.
+     *
+     * @param   layers   list of layers
+     * @param   scale    aux item of interest
+     * @return   scale axis label, or null if nothing suitable could be found
+     */
+    public static String getScaleAxisLabel( PlotLayer[] layers,
+                                            AuxScale scale ) {
+        ValueInfo info = getScaleInfo( layers, scale );
+        if ( info != null ) {
+            String name = info.getName();
+            String unit = info.getUnitString();
+            if ( name != null && unit != null ) {
+                return name + " / " + unit;
+            }
+            else if ( name != null ) {
+                return name;
+            }
+        }
+        return null;
     }
 
     /**
@@ -907,6 +942,9 @@ public class PlotUtil {
             Math.max( 0, (int) Math.round( -Math.log10( epsilon / aval ) ) );
 
         /* Return a formatted string on this basis. */
+        if ( aval <= Double.MIN_NORMAL ) {
+            return "0";
+        }
         if ( aval >= 1e6 || aval <= 1e-4 ) {
             return formatNumber( value, "0.#E0", nsf );
         }
