@@ -13,6 +13,8 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import javax.swing.Icon;
+import uk.ac.starlink.table.DefaultValueInfo;
+import uk.ac.starlink.table.ValueInfo;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
 import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot.Shader;
@@ -39,6 +41,7 @@ import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
 import uk.ac.starlink.ttools.plot2.config.ConfigMeta;
 import uk.ac.starlink.ttools.plot2.config.DoubleConfigKey;
+import uk.ac.starlink.ttools.plot2.config.OptionConfigKey;
 import uk.ac.starlink.ttools.plot2.config.RampKeySet;
 import uk.ac.starlink.ttools.plot2.config.StyleKeys;
 import uk.ac.starlink.ttools.plot2.data.Coord;
@@ -63,6 +66,10 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
     private final boolean transparent_;
     private final boolean reportAuxKeys_;
 
+    /** Weighting coordinate. */
+    private static final FloatingCoord WEIGHT_COORD =
+        FloatingCoord.WEIGHT_COORD;
+
     /** ReportKey for actual X bin extent. */
     public static final ReportKey<Double> XBINWIDTH_KEY =
         createBinWidthReportKey( 'x' );
@@ -85,10 +92,30 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
     /** Config key for Y bin phase. */
     public static final ConfigKey<Double> YPHASE_KEY = createPhaseKey( 'y' );
 
+    /** Config key for combination mode. */
+    public static final ConfigKey<Combiner> COMBINER_KEY =
+        new OptionConfigKey<Combiner>(
+            new ConfigMeta( "combine", "Combine" )
+           .setShortDescription( "Value combination mode" )
+           .setXmlDescription( new String[] {
+               "<p>Defines how values contributing to the same grid cell",
+               "are combined together to produce the value",
+               "assigned to that cell, and hence its colour.",
+               "The combined values are the weights, but if the",
+               "<code>" + WEIGHT_COORD.getInput().getMeta().getShortName()
+                        + "</code> coordinate",
+               "is left blank, a weighting of unity is assumed.",
+               "</p>",
+            } )
+        , Combiner.class, Combiner.getKnownCombiners(), Combiner.MEAN ) {
+        public String getXmlDescription( Combiner combiner ) {
+            return combiner.getDescription();
+        }
+    }.setOptionUsage()
+     .addOptionsXml();
+
     private static final AuxScale SCALE = AuxScale.COLOR;
     private static final RampKeySet RAMP_KEYS = StyleKeys.AUX_RAMP;
-    private static final FloatingCoord WEIGHT_COORD =
-        FloatingCoord.WEIGHT_COORD;
     public static final ConfigKey<Double> TRANSPARENCY_KEY =
         StyleKeys.TRANSPARENCY;
     private static final CoordGroup COORD_GROUP =
@@ -163,7 +190,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         List<ConfigKey> keyList = new ArrayList<ConfigKey>();
         keyList.add( XBINSIZER_KEY );
         keyList.add( YBINSIZER_KEY );
-        keyList.add( StyleKeys.COMBINER );
+        keyList.add( COMBINER_KEY );
         if ( reportAuxKeys_ ) {
             keyList.addAll( Arrays.asList( RAMP_KEYS.getKeys() ) );
         }
@@ -180,7 +207,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         BinSizer ySizer = config.get( YBINSIZER_KEY );
         double xPhase = config.get( XPHASE_KEY ).doubleValue();
         double yPhase = config.get( YPHASE_KEY ).doubleValue();
-        Combiner combiner = config.get( StyleKeys.COMBINER );
+        Combiner combiner = config.get( COMBINER_KEY );
         RampKeySet.Ramp ramp = RAMP_KEYS.createValue( config );
         Scaling scaling = ramp.getScaling();
         float scaleAlpha = 1f - config.get( TRANSPARENCY_KEY ).floatValue();
@@ -320,15 +347,18 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
      * @param  g  graphics context
      * @param  pixer   defines pixel grid
      * @param  binResult  grid data
+     * @param  ctype   combiner type
      * @param  scaler   scales bin values to unit range
      * @param  colorModel  colour map; index zero corresponds to transparency
      * @param  surface   plot surface
      */
     private static void paintBins( Graphics g, GridPixer pixer,
                                    BinList.Result binResult,
+                                   Combiner.Type ctype,
                                    Scaler scaler, IndexColorModel colorModel,
                                    PlanarSurface surface ) {
         int ncolor = colorModel.getMapSize() - 1;
+        double binFactor = pixer.getBinFactor( ctype );
 
         /* Sample bin grid onto output pixel grid. */
         if ( true ) {
@@ -351,7 +381,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 if ( ib >= 0 ) {
                     if ( ib != ib0 ) {
                         ib0 = ib;
-                        double dval = binResult.getBinValue( ib );
+                        double dval = binFactor * binResult.getBinValue( ib );
                         sval = Double.isNaN( dval )
                              ? 0
                              : Math.min( 1 + (int) ( scaler.scaleValue( dval )
@@ -393,7 +423,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 for ( int ix = ixlo; ix <= ixhi; ix++ ) {
                     int ibin = pixer.getBinIndex( ix, iy );
                     assert ibin >= 0;
-                    double dval = binResult.getBinValue( ibin );
+                    double dval = binFactor * binResult.getBinValue( ibin );
                     if ( ! Double.isNaN( dval ) ) {
                         int sval =
                             Math.min( 1 + (int) ( scaler.scaleValue( dval )
@@ -538,6 +568,9 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 public int getCoordIndex() {
                     return icWeight_;
                 }
+                public ValueInfo getAxisInfo( DataSpec dataSpec ) {
+                    return getCombinedInfo( dataSpec );
+                }
                 public void adjustAuxRange( Surface surface, DataSpec dataSpec,
                                             DataStore dataStore, Object[] plans,
                                             Range range ) {
@@ -697,6 +730,8 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         private void extendRange( Range range, PlanarSurface surface,
                                   GridPixer pixer, BinList.Result binResult ) {
             double[][] dlims = surface.getDataLimits();
+            double binFactor =
+                pixer.getBinFactor( gstyle_.combiner_.getType() );
             int[] ixRange = pixer.xgrid_.getBinRange( dlims[ 0 ] );
             int[] iyRange = pixer.ygrid_.getBinRange( dlims[ 1 ] );
             int ixlo = ixRange[ 0 ];
@@ -707,11 +742,35 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 for ( int ix = ixlo; ix <= ixhi; ix++ ) {
                     int ibin = pixer.getBinIndex( ix, iy );
                     assert ibin >= 0;
-                    range.submit( binResult.getBinValue( ibin ) );
+                    range.submit( binFactor * binResult.getBinValue( ibin ) );
                 }
             }
         }
 
+        /**
+         * Returns the metadata for the combined values.
+         *
+         * @param  dataSpec  data specification
+         * @return   metadata for gridded cells
+         */
+        private ValueInfo getCombinedInfo( DataSpec dataSpec ) {
+            final ValueInfo weightInfo;
+            if ( icWeight_ < 0 || dataSpec.isCoordBlank( icWeight_ ) ) {
+                weightInfo = new DefaultValueInfo( "1", Double.class,
+                                                   "Weight unspecified"
+                                                   + ", taken as unity" );
+            }
+            else {
+                ValueInfo[] winfos = dataSpec.getUserCoordInfos( icWeight_ );
+                weightInfo = winfos != null && winfos.length == 1
+                           ? winfos[ 0 ]
+                           : new DefaultValueInfo( "Weight", Double.class );
+            }
+            Unit unit = new Unit( "unit", "unit area", "area", 1,
+                                  "X axis unit * Y axis unit" );
+            return gstyle_.combiner_.createCombinedInfo( weightInfo, unit );
+        }
+
         /**
          * Drawing implementation for the density map.
          */
@@ -756,6 +815,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
             public void paintData( Object plan, Paper paper,
                                    DataStore dataStore ) {
                 final GridPlan gplan = (GridPlan) plan;
+                final Combiner.Type ctype = gstyle_.combiner_.getType();
                 ptype_.placeDecal( paper, new Decal() {
                     public void paintDecal( Graphics g ) {
                         Scaler scaler =
@@ -764,7 +824,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                         IndexColorModel colorModel =
                             PixelImage.createColorModel( gstyle_.shader_,
                                                          true );
-                        paintBins( g, gplan.pixer_, gplan.result_,
+                        paintBins( g, gplan.pixer_, gplan.result_, ctype,
                                    scaler, colorModel, surface_ );
                     }
                     public boolean isOpaque() {
@@ -899,7 +959,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
          * Returns the grid index given the X and Y grid indices.
          *
          * @param   ix  X gridder bin index
-         * @param   iy  Y girdder bin index
+         * @param   iy  Y gridder bin index
          * @return   grid index for bin list, or negative number if off-grid
          */
         int getBinIndex( int ix, int iy ) {
@@ -909,6 +969,17 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                  : -1;
         }
 
+        /**
+         * Returns the area of a bin in data units.
+         * If either axis is logarithmic, this will be a strange quantity.
+         *
+         * @param  ctype  combination type
+         * @return  multiplication for bin values
+         */
+        double getBinFactor( Combiner.Type ctype ) {
+            return ctype.getBinFactor( xgrid_.binWidth_ * ygrid_.binWidth_ );
+        }
+
         /**
          * Indicates whether this pixer is a superset of the given pixer.
          *
