@@ -235,6 +235,62 @@ public class FuncTest extends TestCase {
                                                null ) );
         assertArrayEquals( new int[] { 7, 10, 12 },
                            Arrays.intArray( 7, 10, 12 ) );
+
+        double[] da = new double[] { 10., 11., 12., 13., 14., 15., };
+        int[] ia = new int[] { 10, 11, 12, 13, 14, 15, };
+        String[] sa = new String[] { "Tess", "Armadillo", "Scout", };
+
+        assertArrayEquals( new double[] { 10, 11, 12 },
+                           Arrays.slice(Arrays.array(10,11,12,13), 0, 3) );
+        assertArrayEquals( new double[] { 12, 13 },
+                           Arrays.slice(Arrays.array(10,11,12,13), -2, 999) );
+        assertArrayEquals( new String[] { "A", "B", "C", },
+               Arrays.slice(Arrays.stringArray("A","B","C","D"), 0, 3));
+        assertArrayEquals( new String[] { "C", "D", },
+               Arrays.slice(Arrays.stringArray("A","B","C","D"),-2,999));
+
+        assertArrayEquals( new double[] { 11, 12, 13, 14 },
+                           Arrays.slice( da, 1, -1 ) );
+        assertArrayEquals( new int[] { 10, 11, 12, 13, },
+                           Arrays.slice( ia, -99, -2 ) );
+        assertArrayEquals( new String[ 0 ],
+                           Arrays.slice( sa, 2, 1 ) );
+        assertArrayEquals( new String[] { "Tess", },
+                           Arrays.slice( sa, 0, 1 ) );
+
+        assertArrayEquals( new double[] { 10, 13 },
+                           Arrays.pick(Arrays.array(10,11,12,13), 0, 3) );
+        assertArrayEquals( new double[] { 13, 12, 11 },
+                           Arrays.pick(Arrays.array(10,11,12,13), -1, -2, -3 ));
+        assertArrayEquals( new int[] { 10, 13 },
+                           Arrays.pick(Arrays.intArray(10,11,12,13), 0, 3));
+        assertArrayEquals( new int[] { 13, 12, 11 },
+              Arrays.pick( Arrays.intArray(10,11,12,13),-1,-2,-3));
+        assertArrayEquals( new String[] { "A", "D", },
+              Arrays.pick(Arrays.stringArray("A","B","C","D"), 0, 3 ));
+        assertArrayEquals( new String[] { "D", "C", "B", },
+              Arrays.pick(Arrays.stringArray("A","B","C","D"),-1,-2,-3));
+
+        assertArrayEquals( new double[] { 11, 15, 12 },
+                           Arrays.pick( da, 1, 5, 2 ) );
+        assertArrayEquals( new int[] { 11, 15, 12 },
+                           Arrays.pick( ia, 1, 5, 2 ) );
+        assertArrayEquals( new double[] { 15, 14, 13 },
+                           Arrays.pick( da, -1, -2, -3 ) );
+        try {
+            assertArrayEquals( null, Arrays.pick( da, -99 ) );
+            fail();
+        }
+        catch ( IndexOutOfBoundsException e ) {
+        }
+        try {
+            assertArrayEquals( null, Arrays.pick( ia, 99 ) );
+            fail();
+        }
+        catch ( IndexOutOfBoundsException e ) {
+        }
+        assertArrayEquals( new String[] { "Armadillo", "Scout", },
+                           Arrays.pick( sa, 1, -1 ) );
     }
 
     public void testConversions() {
@@ -608,11 +664,14 @@ public class FuncTest extends TestCase {
             TINY );
 
         assertEquals( Maths.PI / 4.0,
-                      CoordsRadians.hmsToRadians( "03:00:00.0" ) );
+                      CoordsRadians.hmsToRadians( "03:00:00.0" ),
+		      TINY );
         assertEquals( -Maths.PI / 4.0,
-                      CoordsRadians.hmsToRadians( "-03: 0:0" ) );
+                      CoordsRadians.hmsToRadians( "-03: 0:0" ),
+		      TINY );
         assertEquals( CoordsRadians.hmsToRadians( "0 0 1" ),
-                      -CoordsRadians.hmsToRadians( "-0h0m1s" ) );
+                      -CoordsRadians.hmsToRadians( "-0h0m1s" ),
+		      TINY );
 
         assertEquals( "03:00:00",
                       CoordsRadians.radiansToHms( Maths.PI / 4.0 ) );
@@ -686,10 +745,10 @@ public class FuncTest extends TestCase {
         assertEquals( -90, CoordsDegrees.posAngDegrees( 15, 0, -15, 0 ), TINY );
         assertEquals( 0., CoordsDegrees.posAngDegrees( 35, 8, 35, 12 ), TINY );
 
-        assertEquals( 45, CoordsDegrees.hmsToDegrees( "03:00:00.0" ) );
-        assertEquals( -45, CoordsDegrees.hmsToDegrees( "-03: 0:0" ) );
+        assertEquals( 45, CoordsDegrees.hmsToDegrees( "03:00:00.0" ), TINY );
+        assertEquals( -45, CoordsDegrees.hmsToDegrees( "-03: 0:0" ), TINY );
         assertEquals( CoordsDegrees.hmsToDegrees( "0 0 1" ),
-                      -CoordsDegrees.hmsToDegrees( "-0h0m1s" ) );
+                      -CoordsDegrees.hmsToDegrees( "-0h0m1s" ), TINY );
 
         assertEquals( "03:00:00", CoordsDegrees.degreesToHms( 45 ) );
         assertEquals( "12:00:00.000", CoordsDegrees.degreesToHms( -180, 3 ) );
@@ -721,20 +780,34 @@ public class FuncTest extends TestCase {
         for ( int ik = 0; ik < 4; ik++ ) {
             long npix = 12L << 2*ik;
             for ( long ip = 0; ip < npix; ip++ ) {
-                assertEquals( ip, Tilings.healpixRingToNest( ik,
-                                  Tilings.healpixNestToRing( ik, ip ) ) );
-                assertEquals( ip, Tilings.healpixNestToRing( ik,
-                                  Tilings.healpixRingToNest( ik, ip ) ) );
-                assertEquals( Tilings.healpixNestLat( ik, ip ),
-                              Tilings.healpixRingLat( ik,
-                                   Tilings.healpixNestToRing( ik, ip ) ),
-                              1e-6 );
-                assertEquals( Tilings.healpixNestLon( ik, ip ),
-                              Tilings.healpixRingLon( ik,
-                                   Tilings.healpixNestToRing( ik, ip ) ),
-                              1e-6 );
+                checkHealpix( ik, ip );
             }
         }
+        checkHealpix( 2, 23 );
+        checkHealpix( 9, 1110787 );
+    }
+
+    private void checkHealpix( int ik, long ipix ) {
+        assertEquals( ipix, Tilings.healpixRingToNest( ik,
+                            Tilings.healpixNestToRing( ik, ipix ) ) );
+        assertEquals( ipix, Tilings.healpixNestToRing( ik,
+                            Tilings.healpixRingToNest( ik, ipix ) ) );
+        assertEquals( Tilings.healpixNestLat( ik, ipix ),
+                      Tilings.healpixRingLat( ik,
+                           Tilings.healpixNestToRing( ik, ipix ) ),
+                      1e-6 );
+        assertEquals( Tilings.healpixNestLon( ik, ipix ),
+                      Tilings.healpixRingLon( ik,
+                           Tilings.healpixNestToRing( ik, ipix ) ),
+                      1e-6 );
+        assertEquals( ipix,
+                      Tilings.healpixNestIndex( ik,
+                                 Tilings.healpixNestLon( ik, ipix ),
+                                 Tilings.healpixNestLat( ik, ipix ) ) );
+        assertEquals( ipix,
+                      Tilings.healpixRingIndex( ik,
+                                 Tilings.healpixRingLon( ik, ipix ),
+                                 Tilings.healpixRingLat( ik, ipix ) ) );
     }
 
     public void testJELClasses() {
